@@ -63,11 +63,15 @@ import inspect
 import logging
 import os
 import signal
+import stat
 import sys
+import threading
+import time
 import traceback
 
 from oslo_utils import timeutils
 
+from oslo_reports._i18n import _LE
 from oslo_reports._i18n import _LW
 from oslo_reports.generators import conf as cgen
 from oslo_reports.generators import process as prgen
@@ -75,6 +79,7 @@ from oslo_reports.generators import threading as tgen
 from oslo_reports.generators import version as pgen
 from oslo_reports import report
 
+
 LOG = logging.getLogger(__name__)
 
 
@@ -84,7 +89,7 @@ class GuruMeditation(object):
     This class is a base class for Guru Meditation Reports.
     It provides facilities for registering sections and
     setting up functionality to auto-run the report on
-    a certain signal.
+    a certain signal or use file modification events.
 
     This class should always be used in conjunction with
     a Report class via multiple inheritance.  It should
@@ -124,7 +129,8 @@ class GuruMeditation(object):
 
         This method sets up the Guru Meditation Report to automatically
         get dumped to stderr or a file in a given dir when the given signal
-        is received.
+        is received. It can also use file modification events instead of
+        signals.
 
         :param version: the version object for the current product
         :param service_name: this program name used to construct logfile name
@@ -140,16 +146,52 @@ class GuruMeditation(object):
             cls._setup_signal(signum, version, service_name, log_dir)
             return
 
-        if hasattr(signal, 'SIGUSR1'):
-            # TODO(dims) We need to remove this in the "O" release cycle
-            LOG.warning(_LW("Guru mediation now registers SIGUSR1 and SIGUSR2 "
-                            "by default for backward compatibility. SIGUSR1 "
-                            "will no longer be registered in a future "
-                            "release, so please use SIGUSR2 to "
-                            "generate reports."))
-            cls._setup_signal(signal.SIGUSR1, version, service_name, log_dir)
-        if hasattr(signal, 'SIGUSR2'):
-            cls._setup_signal(signal.SIGUSR2, version, service_name, log_dir)
+        if conf and conf.oslo_reports.file_event_handler:
+            cls._setup_file_watcher(
+                conf.oslo_reports.file_event_handler,
+                conf.oslo_reports.file_event_handler_interval,
+                version, service_name, log_dir)
+        else:
+            if hasattr(signal, 'SIGUSR1'):
+                # TODO(dims) We need to remove this in the "O" release cycle
+                LOG.warning(_LW("Guru meditation now registers SIGUSR1 and "
+                                "SIGUSR2 by default for backward "
+                                "compatibility. SIGUSR1 will no longer be "
+                                "registered in a future release, so please "
+                                "use SIGUSR2 to generate reports."))
+                cls._setup_signal(signal.SIGUSR1,
+                                  version, service_name, log_dir)
+            if hasattr(signal, 'SIGUSR2'):
+                cls._setup_signal(signal.SIGUSR2,
+                                  version, service_name, log_dir)
+
+    @classmethod
+    def _setup_file_watcher(cls, filepath, interval, version, service_name,
+                            log_dir):
+
+        st = os.stat(filepath)
+        if not bool(st.st_mode & stat.S_IRGRP):
+            LOG.error(_LE("Guru Meditation Report does not have read "
+                          "permissions to '%s' file."), filepath)
+
+        def _handler():
+            mtime = time.time()
+            while True:
+                try:
+                    stat = os.stat(filepath)
+                    if stat.st_mtime > mtime:
+                        cls.handle_signal(version, service_name, log_dir, None)
+                        mtime = stat.st_mtime
+                except OSError:
+                    msg = ("Guru Meditation Report cannot read " +
+                           "'{0}' file".format(filepath))
+                    raise IOError(msg)
+                finally:
+                    time.sleep(interval)
+
+        th = threading.Thread(target=_handler)
+        th.daemon = True
+        th.start()
 
     @classmethod
     def _setup_signal(cls, signum, version, service_name, log_dir):
